import json
import logging
from abc import ABC
from requests import request
from urllib.parse import urlencode

class APIRequestor(ABC):

    # URL Base para requests. Não informar '/' ao final.
    base_api_url:str

    base_payload:dict
    base_headers:dict

    def mountURL(self,url : str):
        URL = '%s/%s' % (self.base_api_url,url)
        return URL

    def mountHeaders(self,headers:dict={}):
        req_headers = {
            **{
                k:' '.join(v) if isinstance(v,list) else v \
                    for k,v in self.base_headers.items()
            },
            **headers
        }

        logging.info(req_headers)

        return req_headers

    def mountPayload(self,payload:dict={}):
        
        req_payload = self.base_payload or {}

        if payload: 
            req_payload = {
                **req_payload,
                **payload
            }

        logging.info(payload)
        return req_payload

    def toURLQueryString(self,query:dict={}):
        
        if query:
            return '?%s' % urlencode(query)

        return ''

    def GET(self, url, query:dict={}):

        request_url = self.mountURL(url + self.toURLQueryString(query))
        request_headers = self.mountHeaders()
        request_payload = self.mountPayload()

        return request(
            "GET", 
            url=request_url,
            headers=request_headers,
            data=request_payload,
            files=[]
        )

    def POST(self, url, data:dict={}):
        print(url)
        jsonData = self.mountPayload(data)

        return request(
            "POST", 
            url=self.mountURL(url), 
            headers=self.mountHeaders(), 
            json=jsonData, 
        )
